/* copy/copyStream, from usrLib.c  */
/* Copyright 1984-1993 Wind River Systems, Inc. */
#include "copyright_wrs.h"

#include <vxWorks.h>
#include <ioLib.h>
#include <flags.h>

/* KENC, pain that I have to use this RAM... */
static char buffer[2048];				/* big enough for most files? */

static STATUS copyStreams(int inFd, int outFd);


/*******************************************************************************
*
* copy - copy <in> (or 'stdin') to <out> (or 'stdout')
*
* This command copies from the input file to the output file, until an
* end-of-file is reached.
*
*/
STATUS xcopy
(
    char *in,	/* name of file to read  (if NULL assume stdin)  */
    char *out 	/* name of file to write (if NULL assume stdout) */
)
{
    int inFd =  in ?  open(in, O_RDONLY, 0) : STD_IN;
    int outFd = out ? creat(out, O_WRONLY)  : STD_OUT;
    int status;

    if (inFd < OK) {
	if (storagePrint) printf("Can't open %s\n", in);
	return ERROR;
    }
    if (outFd < OK) {
	close (inFd);
	if (storagePrint) printf("Can't write to %s\n", out);
	return ERROR;
    }

    status = copyStreams(inFd, outFd);

    close(inFd);
    close(outFd);

    return status;
}


/*******************************************************************************
*
* copyStreams - copy from/to specified streams
*
* This command copies from the stream identified by <inFd> to the stream
* identified by <outFd> until an end of file is reached in <inFd>.
* This command is used by copy().
*
* RETURNS: OK, or ERROR if there is an error reading from <inFd> or writing to <outFd>.
*
*/
static STATUS copyStreams
(
    int inFd,		/* file descriptor of stream to copy from */
    int outFd 		/* file descriptor of stream to copy to */
)
{
    int totalBytes = 0;
    int nbytes;

    while ((nbytes = fioRead(inFd, buffer, sizeof(buffer))) > 0) {
	if (write(outFd, buffer, nbytes) != nbytes) {
	    if (storagePrint) printf("copyStreams: error writing file.\n");
	    return ERROR;
	}

	totalBytes += nbytes;

	/* fioRead returns with less than full buffer only if EOF */
	if (nbytes != sizeof(buffer)) break;	/* EOF */
    }

    if (nbytes < 0) {
	if (storagePrint) printf("copyStreams: error reading file after copying %d bytes.\n", totalBytes);
	return ERROR;
    }

    return OK;
}
