/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Ken Coulson, Tektronix, Inc. <kenc@lollipop.cna.tek.com>
 */ 
#include <stdioLib.h>
#include <taskLib.h>
#include <mrgcl340.h>
#include <m68340.h>
#include <flags.h>
#include "flash.h"
#include "packer.h"


#if 0
/*
    Reset all devices.
*/
void flashResetAll(void)
{
    flashReset(FLASH0START);
    flashReset(FLASH1START);
    flashReset(FLASH2START);
#ifndef MRGCL1
    flashReset(FLASH3START);
    flashReset(FLASH4START);
#endif
}
#endif


/*
    Reset individual block (two devices).
*/
void flashReset(Bit16* start)
{
    register volatile int delay;

    *start = 0xffff;
    *start = 0xffff;
    *start = 0x0000;
    for (delay=3; delay; --delay) {};
}


#if 0
/*
    Identify part.
    Assumes all blocks are the same type.
*/
int flashIdentify(int device)
{
    static Bit16 manufacturer = 0;	/* FLASH manufacturer */
    static Bit16 devicecode = 0;	/* FLASE device code */
    static Bit32 devicetype = 0;	/* combination of the two */
    Bit16* start;
    int delay;

    /*
    if (devicetype) return devicetype;
    */

    switch (device) {
	case FLASH0:
	    start = FLASH0START;
	    break;
	case FLASH1:
	    start = FLASH1START;
	    break;
	case FLASH2:
	    start = FLASH2START;
	    break;
	case FLASH3:
	    start = FLASH3START;
	    break;
	case FLASH4:
	    start = FLASH4START;
	    break;
	default:
	    printf("flashIdentify: illegal device number %d\n", device);
	    return 0;
    }

    flashVpp(ON);
    *start = 0xffff;
    *start = 0xffff;
    *start = 0x0000;
    for (delay=3; delay!=0; --delay);
    *start = 0x9090;
    manufacturer = *start++;
    devicecode = *start;
    *start = 0xffff;
    *start = 0xffff;
    *start = 0x0000;
    printf("flashIdentify: device %d manufacturer %04x\n", device, manufacturer);
    printf("flashIdentify: device %d device code %04x\n", device, devicecode);
    devicetype = (manufacturer << 16) | devicecode;
    printf("flashIdentify: device %d device type %08x\n", device, devicetype);
    flashVpp(OFF);
    return devicetype;
}
#endif


/*

*/
void flashVpp(int state)
{
    if (state == ON) {
	*(M340_DUART_ROP) |= 0x10;		/* Vpp on */
	taskDelay(3);				/* delay at least 10 msec */
    }
    else {
	*(M340_DUART_SOP) |= 0x10;     		/* Vpp off */
    }
}


#if 0
/*
    Don't forget the count is the number of words!
*/
int flashProgram(Bit16* src, Bit16* dest, unsigned int count, int debug) 
{
    register Bit16* address;
    int status = 0;

#if 0
    if (count > FLASHSIZE) {
	printf("flashProgram: src to big %d (max %d)\n", count, FLASHSIZE);
	return 0;
    }
#endif

    flashVpp(ON);
    flashResetAll();
    for (address=dest; count > 0; --count) {
	if (!(status = flashProgramWord(address++, *src++))) break;
	if (debug && !(count & 0x0ff)) printf("%06x\r", count);
    }
    if (debug) printf("\n");
    flashVpp(OFF);
    return status;
}
#endif


/*
    Program a single word.
    Assuming you have Vpp on already.

*/
int flashProgramWord(Bit16* address, Bit16 data)
{
    int attempt;
    register volatile int delay;
    int devicetype;

    if (flashPrint) printf("pgm *%x = %x\n", address, data);
    if (*address == data) return 1;

    /*devicetype = flashIdentify(FLASH0);*/
    devicetype = INTEL;
    if (devicetype == INTEL) {
	/*printf("INTEL\n");*/
	for (attempt=1; attempt<50; ++attempt) {
	    *address = 0x4040;			/* write mode */
	    *address = data;			/* actual write */
	    for (delay=20; delay!=0; --delay);	/* delay 10 usec */
	    *address = 0xc0c0;			/* verify mode (terminates write mode) */
	    for (delay=10; delay!=0; --delay);	/* delay 6 usec */
	    if (*address == data) break;
	}
    }
#if 0
    else if (devicetype == AMD) {
	printf("AMD\n");
	*address = 0x5050;			/* write mode */
	*address = data;			/* actual write */
	for (attempt=1000; attempt; --attempt)
	    if (!((*address & 0x8080) ^ (data & 0x8080))) break;
	printf("flashErase: data poll done (%d)\n", attempt);
    }
    else {
	printf("flashProgramWord: unknown device type %x\n", devicetype);
	return 0;
    }
#endif

    *address = 0xffff;
    *address = 0xffff;
    *address = 0x0000;
    if (!(*address == data)) {
	/* KENC system error */
	if (storagePrint) printf("flashProgramWord: writing %x to %x failed. read of device is %x\n", data, address, *address);
	return 0;
    }
    return 1;
}


#if 0
/*

*/
int flashEraseAll(void)
{
    if (!flashErase(FLASH0)) return 0;	/* these are code */
    if (!flashErase(FLASH1)) return 0;
    if (!flashErase(FLASH2)) return 0;
    if (!flashErase(FLASH3)) return 0;	/* these are storage */
    if (!flashErase(FLASH4)) return 0;
    return 1;
}
#endif


/*

*/
int flashErase(int block)
{
    Bit16* start;
    unsigned int size;
    float floatsize;
    int attempt;
    register volatile int delay;
    register Bit16* address;
    register Bit16* lastaddress;
    register Bit16* endaddress;
    register unsigned int which;
    int devicetype;
    int dummy;
    int status = 0;

    switch (block) {
	case FLASH0:
	    start = FLASH0START;
	    size = FLASH0BLOCKSIZE;
	    break;
	case FLASH1:
	    start = FLASH1START;
	    size = FLASH1BLOCKSIZE;
	    break;
	case FLASH2:
	    start = FLASH2START;
	    size = FLASH2BLOCKSIZE;
	    break;
#ifndef MRGCL1
	case FLASH3:
	    start = FLASH3START;
	    size = FLASH3BLOCKSIZE;
	    break;
	case FLASH4:
	    start = FLASH4START;
	    size = FLASH4BLOCKSIZE;
	    break;
#endif
	default:
	    if (storagePrint) printf("flashErase: illegal block number %d\n", block);
	    return 0;
    }

    address = start;
    which = size;
    floatsize = (float)size;

    if (storagePrint) printf("flashErase: block %d at %x size %d\n", block, start, size);

    do {
	if (*address++ != 0xffff) break;
    } while (--which);
    if (which == 0) { 
	if (storagePrint) printf("flashErase: block %d is blank\n", block);
	return 1;
    }

    address = start;
    which = size;
    /*devicetype = flashIdentify(FLASH0);*/
    devicetype = INTEL;
    flashVpp(ON);
    if (devicetype == INTEL) {
	if (storagePrint) printf("flashErase: block %d not blank.  Filling with 0x00.\n", block);
	do {
	    if (!flashProgramWord(address++, 0)) {
		flashVpp(OFF);
		return 0;
	    }
	    if (!(which & 0x07ff)) {
		if (storagePrint) printf("%06x\r", which);
		/* use 90% here, other 10% below */
		PackPartPercentDone = (size*90 - which*90) / size;
	    }
	} while (--which);
	if (storagePrint) printf("%06x\ndone.\n", 0);
	lastaddress  = start;
	endaddress = start + (size/2);
	for (attempt=0; attempt<=3000; ++attempt) {
	    if (storagePrint) printf("\tflashErase: Erase attempt %d\r", attempt+1);
	    *start = 0x2020;
	    *start = 0x2020;
	    for (delay=5000; delay!=0; --delay) {};		/* use 50 for INTEL! */
	    for (address=lastaddress; address<endaddress; ++address) {
		*address = 0xa0a0;
		for (delay=15; delay!=0; --delay) {};
		if (*address != 0xffff) {
		    lastaddress = address;
		    break;
		}
		if (!((int)address & 0x07ff)) {
		    if (storagePrint) printf("%06x\r", size - ((int)address - (int)start));
		    /* 10% here other 90% above */
		    PackPartPercentDone = 90 + (((int)address*10 - (int)start*10) / size);
		}
	    }
	    status = (address == endaddress) ? 1 : 0;
	    if (status) break;
	}
	if (storagePrint) printf("%06x\n", 0);
    }
#if 0
    else if (devicetype == AMD) {
	printf("AMD\n");
        *address = 0x3030;
        *address = 0x3030;
	for (attempt=5000; attempt; --attempt) {
	    dummy = 0;
	    if ((*address & 0x8080) == 0x8080) break;
	}
	printf("flashErase: data poll done (%d)\n", attempt);
	for (address=start; address < start+size; ++address)
	    if (*address != 0xffff) break;
	status = (address == start+size) ? 1 : 0;
    }
    else {
	printf("flashErase: unknown device type %x\n", devicetype);
	status = 0;
    }
#endif

    if (storagePrint) printf("Erase of block %d %s\n\n", block, (status) ? "passed" : "failed");
    flashVpp(OFF);
    return status;
}


#if 0
/*

*/
int flashEraseCheck(int block)
{
    int status = 1;
    unsigned int size;
    register Bit16* p;

    switch (block) {
	case FLASH0:
	    p = FLASH0START;
	    size = FLASH0BLOCKSIZE;
	    break;
	case FLASH1:
	    p = FLASH1START;
	    size = FLASH1BLOCKSIZE;
	    break;
	case FLASH2:
	    p = FLASH2START;
	    size = FLASH2BLOCKSIZE;
	    break;
	case FLASH3:
	    p = FLASH3START;
	    size = FLASH3BLOCKSIZE;
	    break;
	case FLASH4:
	    p = FLASH4START;
	    size = FLASH4BLOCKSIZE;
	    break;
	default:
	    if (storagePrint) printf("flashEraseCheck: illegal block number %d\n", block);
	    return 0;
    }

    do {} while ((*p++ == 0xffff) && (--size));

    if (size == 0) {
	return 1;
    }
    else {
	--p;
	if (storagePrint) printf("flashEraseCheck: Not erased. *%x = %x\n", p, *p);
	return 0;
    }
}
#endif


int flashEnable(int state)
{
    int status;
    word data = (state) ? FLASHMAGIC : 0;

    flashVpp(ON);
    status = flashProgramWord(FLASHSTART, data);
    flashVpp(OFF);
    return status;
}


/*
	Get address and size for a flash part used
	for waveform storage (i.e. psuedo-disk).
*/
char flashPartSize(char disk, Bit8** start, unsigned int* size)
{
    switch (disk) {
	case 0:
#ifdef MRGCL1
	    *start = (Bit8*)FLASH2START;
	    *size = FLASH2BLOCKSIZE;
#else
	    *start = (Bit8*)FLASH3START;
	    *size = FLASH3BLOCKSIZE;
#endif
	    break;

	case 1:
#ifdef MRGCL1
	    *start = (Bit8*)FLASH2START + 1;
	    *size = FLASH2BLOCKSIZE;
#else
	    *start = (Bit8*)FLASH3START + 1;
	    *size = FLASH3BLOCKSIZE;
#endif
	    break;

	case 2:
#ifdef MRGCL1
	    return 0;
#else
	    *start = (Bit8*)FLASH4START;
	    *size = FLASH4BLOCKSIZE;
#endif
	    break;

	case 3:
#ifdef MRGCL1
	    return 0;
#else
	    *start = (Bit8*)FLASH4START + 1;
	    *size = FLASH4BLOCKSIZE;
#endif
	    break;
	default:
	    if (storagePrint) printf("flashPartSize: illegal disk number %d\n", disk);
	    return 0;
    }
    return 1;
}


/*
    Program a single byte.
    Assuming you have Vpp on already.

*/
int flashProgramByte(Bit8* address, Bit8 data)
{
    int attempt;
    register volatile int delay;
    int devicetype;

    if (flashPrint) printf("pgm *%x = %x\n", address, data);
    if (*address == data) return 1;

    /*devicetype = flashIdentify(FLASH0);*/
    devicetype = INTEL;
    if (devicetype == INTEL) {
	/*printf("INTEL\n");*/
	for (attempt=1; attempt<50; ++attempt) {
	    *address = 0x40;			/* write mode */
	    *address = data;			/* actual write */
	    for (delay=20; delay!=0; --delay);	/* delay 10 usec */
	    *address = 0xc0;			/* verify mode (terminates write mode) */
	    for (delay=10; delay!=0; --delay);	/* delay 6 usec */
	    if (*address == data) break;
	}
    }
#if 0
    else if (devicetype == AMD) {
	printf("AMD\n");
	*address = 0x50;			/* write mode */
	*address = data;			/* actual write */
	for (attempt=1000; attempt; --attempt)
	    if (!((*address & 0x80) ^ (data & 0x80))) break;
	printf("flashErase: data poll done (%d)\n", attempt);
    }
    else {
	printf("flashProgramWord: unknown device type %x\n", devicetype);
	return 0;
    }
#endif

    *address = 0xff;
    *address = 0xff;
    *address = 0x00;
    if (!(*address == data)) {
	/* KENC system error */
	if (storagePrint) printf("flashProgramByte: writing %x to %x failed. read of device is %x\n", data, address, *address);
	return 0;
    }
    return 1;
}

/*
    Program an array of bytes into a single device
*/
int flashWritePart(char disk, Bit8* faddr, Bit8* bufAddr, int count)
{
    Bit8 *baseAddress, *addr;
    unsigned int size;
    int nbytes = 0;

    if (!flashPartSize(disk, &baseAddress, &size))
	return 0;

    flashVpp(ON);
    addr = baseAddress + ((unsigned int)faddr * 2);
    for (; nbytes<count; addr+=2, nbytes++) {
	if (!flashProgramByte(addr, *bufAddr++))
	    return nbytes;
    }
    flashVpp(OFF);

    return nbytes;
}

/*
    Read an array of bytes from a single device
*/
int flashReadPart(char disk, Bit8* faddr, Bit8* bufAddr, int count)
{
    Bit8 *baseAddress, *addr;
    unsigned int size;
    int nbytes = 0;

    if (!flashPartSize(disk, &baseAddress, &size))
	return 0;

    addr = baseAddress + ((unsigned int)faddr * 2);
    while (count--) {
	*bufAddr++ = *addr++;
	addr++;
	nbytes++;
    }

    return nbytes;
}

/*
    erase a single flash part
    returns vxWorks style STATUS: 0 = OK, -1 = ERROR
*/
int flashErasePart(char disk)
{
    Bit8* start;
    unsigned int size;
    int attempt;
    register volatile int delay;
    register Bit8* address;
    register Bit8* lastaddress;
    register Bit8* endaddress;
    register unsigned int which;
    int devicetype;
    int dummy;
    int status = 0;

    if (!flashPartSize(disk, &start, &size))
	return -1;

    address = start;
    which = size;
    do {
	if (*address++ != 0xff) break;
	address++;
    } while (--which);
    if (which == 0) { 
	if (storagePrint) printf("flashErase: Disk %d is blank\n", disk);
	return 0;
    }

    address = start;
    which = size;
    /*devicetype = flashIdentify(FLASH0);*/
    devicetype = INTEL;
    flashVpp(ON);
    if (devicetype == INTEL) {
	/*printf("AMD/INTEL\n");*/
	if (storagePrint) printf("flashErase: Disk %d not blank.  Filling with 0x00.\n", disk);
	do {
	    if (!flashProgramByte(address++, 0)) {
		flashVpp(OFF);
		return -1;
	    }
	    address++;
	    if (!(which & 0x07ff)) {
		if (storagePrint) printf("%06x\r", which);
		/* use 90% here, other 10% below */
		PackPartPercentDone = ((size - which)*90) / size;
	    }
	} while (--which);
	if (storagePrint) printf("%06x\ndone.\n", 0);
	lastaddress  = start;
	endaddress = start + size;
	for (attempt=0; attempt<=3000; ++attempt) {
	    if (storagePrint) printf("\tflashErase: Erase attempt %d\r", attempt+1);
	    *start = 0x20;
	    *start = 0x20;
	    for (delay=5000; delay!=0; --delay) {};	/* use 50 for INTEL! */
	    for (address=lastaddress; address<endaddress; address+=2) {
		*address = 0xa0;
		for (delay=15; delay!=0; --delay) {};
		if (*address != 0xff) {
		    lastaddress = address;
		    break;
		}
		if (!((int)address & 0x7ff)) {
		    if (storagePrint) printf("%06x\r", size - ((int)address - (int)start));
		    /* 10% here other 90% above */
		    PackPartPercentDone = 90 + ((((int)address - (int)start)*10) / size);
		}
	    }
	    status = (address == endaddress) ? 1 : 0;
	    if (status) break;
	}
	if (storagePrint) printf("%06x\n", 0);
    }
#if 0
    else if (devicetype == AMD) {
	printf("AMD\n");
        *address = 0x30;
        *address = 0x30;
	for (attempt=5000; attempt; --attempt) {
	    dummy = 0;
	    if ((*address & 0x80) == 0x80) break;
	}
	printf("flashErase: data poll done (%d)\n", attempt);
	for (address=start; address < start+size; address+=2)
	    if (*address != 0xff) break;
	status = (address == start+size) ? 1 : 0;
    }
    else {
	printf("flashErase: unknown device type %x\n", devicetype);
	status = 0;
    }
#endif

    if (storagePrint) printf("Erase of disk %d %s\n\n", disk, (status) ? "passed" : "failed");
    flashVpp(OFF);
    return (status ? 0 : -1);
}

