/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Ken Coulson, Tektronix, Inc. <kenc@lollipop.cna.tek.com>
 */ 
typedef int		boolean;
typedef	unsigned int	Bit32;
typedef	unsigned int	sBit32;
typedef	unsigned short	Bit16;
typedef	unsigned char	Bit8;
typedef unsigned short	word;
typedef unsigned char	byte;

void flashResetAll(void);
void flashReset(Bit16* start);
int flashIdentify(int device);
void flashVpp(int state);
int flashProgram(Bit16* src, Bit16* dest, unsigned int count, int debug);
int flashProgramWord(Bit16* address, Bit16 data);
int flashEraseAll(void);
int flashErase(int device);
int flashEraseCheck(int device);
int flashLoad(int size, int baud);
int flashEnable(int state);
int flashWritePart(char disk, Bit8* flashAddr, Bit8* bufAddr, int count);
	/* returns number of bytes written */
int flashReadPart(char disk, Bit8* flashAddr, Bit8* bufAddr, int count);
	/* returns number of bytes read */
int flashErasePart(char disk);
	/* returns OK/ERROR (0/-1) */
char flashPartSize(char disk, Bit8** start, unsigned int *size);
	/* sets starting address and size of the given disk (part) */
	/* returns 1 on success, 0 on failure */


/*
  MRGCL1's are 360k by 16.  MRGCL1+'s are 720k by 16.
  6 devices total.  A block is two devices wide.
*/
#define FLASH0		0
#define FLASH1		1
#define FLASH2		2
#define FLASH3		3
#define FLASH4		4

#define BIG_FLASH

#if defined(BIG_FLASH) && !defined(MRGCL1_WENDYG)
#define FLASH0START		((Bit16*) 0xe00000)
#define FLASH1START		((Bit16*) 0xe80000)
#define FLASH2START		((Bit16*) 0xf00000)
#define FLASH3START		((Bit16*) 0xc00000)
#define FLASH4START		((Bit16*) 0xc80000)
#define FLASH0BLOCKSIZE		((unsigned) 0x40000)	/* 256 Kwords (2 devices) */
#define FLASH1BLOCKSIZE		((unsigned) 0x40000)	/* 256 Kwords (2 devices) */
#define FLASH2BLOCKSIZE		((unsigned) 0x40000)	/* 256 Kwords (2 devices) */
#define FLASH3BLOCKSIZE		((unsigned) 0x40000)	/* 256 Kwords (2 devices) */
#define FLASH4BLOCKSIZE		((unsigned) 0x40000)	/* 256 Kwords (2 devices) */
#else
#define FLASH0START		((Bit16*) 0xe00000)
#define FLASH1START		((Bit16*) 0xe40000)
#define FLASH2START		((Bit16*) 0xe80000)
#define FLASH0BLOCKSIZE		((unsigned) 0x20000)	/* 128 Kwords * (2 devices) */
#define FLASH1BLOCKSIZE		((unsigned) 0x20000)	/* 128 Kwords * (2 devices) */
#define FLASH2BLOCKSIZE		((unsigned) 0x20000)	/* 128 Kwords * (2 devices) */
#endif

#define FLASHSTART		FLASH0START
#define FLASHSIZE		((unsigned) (FLASH0BLOCKSIZE + FLASH1BLOCKSIZE + FLASH2BLOCKSIZE))

#define MANUFACTURER_AMD	0x01
#define DEVICE_ID_AMD		0x2a
#define AMD			0x01012a2a
#define MANUFACTURER_INTEL	0x89
#define DEVICE_ID_INTEL		0xb4
#define INTEL			0x8989b4b4

#define OFF	0
#define ON	1

#define FLASHMAGIC		((Bit16) 0x1234)
