/* $Id: flashDrv.c,v 1.4 1993/09/07 21:48:56 kenc Exp $ */
/* flashDrv.c - Flash ROM disk driver */
/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Bill Randle, Tektronix, Inc. <billr@saab.cna.tek.com>
 */ 

/* Based on "ramDrv.c", which is: */
/* Copyright 1984-1992 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
xxx,02aug93,bcr  converted to flash ROM driver
02k,20jan93,jdi  documentation cleanup.
02j,21oct92,jdi  removed mangen SECTION designation.
02i,18jul92,smb  Changed errno.h to errnoLib.h.
02h,04jul92,jcf  scalable/ANSI/cleanup effort.
02g,26may92,rrr  the tree shuffle
02f,04oct91,rrr  passed through the ansification filter
		  -changed READ, WRITE and UPDATE to O_RDONLY O_WRONLY O_RDWR
		  -changed copyright notice
02e,11apr91,jdi  documentation cleanup; doc review by kdl.
02d,19oct90,kdl  changed "msdos" references to "dosFs"; changed "sector"
		 to "block" throughout.
02c,30jul90,dnw  coerced malloc to (char*) to eliminate warning
02b,12jul90,kdl  added bd_statusChk field to BLK_DEV.
02a,15may90,kdl  documentation.
01z,04may90,kdl  added BLK_DEV fields (bd_mode, bd_readyChanged);
		 added FIODISKFORMAT.
01y,23mar90,kdl  changed BLK_DEV field names (blkIo.h).
01x,05mar90,kdl  changed to support multiple file systems.
01w,21feb90,dab  added DEFAULT_DISK_SIZE for ramMkfs().
01v,08jul88,jcf  lint.
01u,06jun88,dnw  changed rtLib to rt11Lib.
01t,30may88,dnw  changed to v4 names.
01s,06nov87,ecs  documentation.
01r,04aug87,gae  made ramClose() return STATUS.
		 Changed "static" stuff in ramDrv().
01q,25mar87,jlf  documentation
01p,13mar87,gae  documentation and fixed/improved version 01o.
01o,11mar87,gae  changed raw device open to not use "-" any more.
		 made ramMkfs() return STATUS not (char *).
		 made ramDrv() be a NOP on successive calls.
01n,17feb87,gae  fixed rtDevInit() call.
01m,04feb87,llk  changed call to rtDevInit.  Specifies number of files.
01l,21dec86,dnw  changed to not get include files from default directories.
01k,05sep86,jlf  documentation
01j,15aug86,ecs  renamed ramIoctrl to ramIoctl, changed to call rtIoctl.
01i,02jul86,jlf  oops.
01h,02jul86,jlf  documentation.
01g,03mar86,jlf  changed ioctrl calls to ioctl.
01f,11nov85,jlf  made ramOpen, ramCreate, and ramDelete do a rtReadyChange.
		   The is a kludge so when multiple CPU's share a RAM
		   device, the directory will be reread when any of them
		   try to create, delete, or open a file.
01e,19sep85,jlf  got rid of checkMount, and removed member `mounted' from
		 dev descriptor.
01d,12sep85,jlf  updated calling sequence to rtDevInit.
01c,14aug85,dnw  added ramMkfs.
01b,21jul85,jlf  documentation
01a,24may85,jlf  written, by modifying v03k of fd208Drv.c.
*/

/*
This driver emulates a disk driver, but actually keeps all data in memory.
The memory location and size are specified when the "disk" is created.
The RAM disk feature is useful when data must be preserved between boots of
VxWorks or when sharing data between CPUs.

USER-CALLABLE ROUTINES
Most of the routines in this driver are accessible only through the I/O system.
Two routines, however, can be called directly by the user.  The first, ramDrv(),
provides no real function except to parallel the initialization function found
in true disk device drivers.  A call to ramDrv() is not required to use the RAM
disk driver.  However, the second routine, ramDevCreate(), must be called 
directly to create RAM disk devices.

Once the device has been created, it must be associated with a name and
file system (dosFs, rt11Fs, or rawFs).  This is accomplished by passing
the value returned by ramDevCreate(), a pointer to a block device
structure, to the file system's device initialization routine or
make-file-system routine.  See the manual entry ramDevCreate() for a
more detailed discussion.

IOCTL FUNCTIONS
The Flash driver is called in response to ioctl() codes in the same manner
as a normal disk driver.  When the file system is unable to handle a
specific ioctl() request, it is passed to the flashDrv driver.  Although
there is no physical device to be controlled, flashDrv does handle a
FIODISKFORMAT request.  All other ioctl()
requests return an error and set the task's `errno' to S_ioLib_UNKNOWN_REQUEST.

SEE ALSO: dosFsDevInit(), dosFsMkfs(), rt11FsDevInit(), rt11FsMkfs(),
rawFsDevInit(),
.pG "I/O System, Local File Systems"

LINTLIBRARY
*/

#include "vxWorks.h"
#include "blkIo.h"
#include "ioLib.h"
#include "iosLib.h"
#include "errnoLib.h"
#include "newstring.h"
#include "stdlib.h"
#include "stdioLib.h"

#include "flash.h"
#include "packer.h"


#define DEFAULT_DISK_SIZE  (512*512)
#define DEFAULT_SEC_SIZE   512


typedef struct		/* FLASH_DEV - Flash ROM disk device descriptor */
    {
    BLK_DEV	 flash_blkdev;	/* generic block device structure */
    char	 flash_device;	/* flash device (part) used for flash disk */
    int		 flash_blkOffset;	/* block offset of this device from device start */
    } FLASH_DEV;



/* FORWARD DECLARATIONS */

LOCAL STATUS flashIoctl ();
LOCAL STATUS flashBlkRd ();
LOCAL STATUS flashBlkWrt ();


/*******************************************************************************
*
* flashDrv - prepare a RAM disk driver for use (optional)
*
* This routine performs no real function, except to provide compatibility
* with earlier versions of flashDrv and to parallel the initialization
* function found in true disk device drivers.  It also is used in
* usrConfig.c to link in the Flash disk driver when building
* VxWorks.  Otherwise, there is no need to call this routine before using
* the RAM disk driver.
*
* RETURNS: OK, always.
*/

STATUS flashDrv ()

    {
    return (OK);			/* no initialization required */
    }

/*******************************************************************************
*
* flashDevCreate - create a Flash disk device
*
* This routine creates a Flash disk device.
*
* The <flashDevice> parameter should be the device of the flash part.
*
* The <bytesPerBlk> parameter specifies the size of each logical block
* on the Flash disk.  If <bytesPerBlk> is zero, 512 is used.
*
* The <blksPerTrack> parameter specifies the number of blocks on each
* logical track of the Flash disk.  If <blksPerTrack> is zero, the count of
* blocks per track is set to <nBlocks> (i.e., the disk is defined
* as having only one track).
*
* The <nBlocks> parameter specifies the size of the disk, in blocks.
* If <nBlocks> is zero, a default size is used.  The default is calculated
* using a total disk size of either 262,144 bytes (2M bits).  This default
* disk size is then divided by <bytesPerBlk> to determine the number
* of blocks.
*
* The <blkOffset> parameter specifies an offset, in blocks, from the start
* of the device to be used when writing or reading the Flash disk.  This
* offset is added to the block numbers passed by the file system during
* disk accesses.  (VxWorks file systems always use block numbers beginning
* at zero for the start of a device.)  Normally, <blkOffset> is 0.
*
* FILE SYSTEMS:
* Once the device has been created, it must be associated with a name and a
* file system (dosFs, rt11Fs, or rawFs).  This is accomplished using the
* file system's device initialization routine or make-file-system routine,
* e.g., dosFsDevInit() or dosFsMkfs().  The flashDevCreate() call returns a
* pointer to a block device structure (BLK_DEV).  This structure contains
* fields that describe the physical properties of a disk device and specify
* the addresses of routines within the flashDrv driver.  The BLK_DEV structure
* address must be passed to the desired file system (dosFs, rt11Fs or rawFs)
* via the file system's device initialization or make-file-system routine.
* Only then is a name and file system associated with the device, making it
* available for use.
*
* EXAMPLE:
* In the following example, a 256-Kbyte Flash disk is created with
* starting address FLASH0_START, 512-byte blocks, a single track, and no
* block offset.  The device is then initialized for use with dosFs and assigned
* the name "DEV1:":
* .CS
*     BLK_DEV *pBlkDev;
*     DOS_VOL_DESC *pVolDesc;
*
*     pBlkDev = flashDevCreate (FLASH0,  512,  512,  512,  0);
*     pVolDesc = dosFsMkfs ("DEV1:", pBlkDev);
* .CE
* The dosFsMkfs() routine calls dosFsDevInit() with default parameters and
* initializes the file system on the disk by calling ioctl() with the
* \%FIODISKINIT function.
*
* If the Flash disk memory already contains a disk image previously created,
* the first argument to flashDevCreate() should be the address in memory, and
* the formatting parameters -- <bytesPerBlk>, <blksPerTrack>, <nBlocks>, and
* <blkOffset> -- must be identical to those used when the image was
* created.  For example:
* .CS
*     pBlkDev = flashDevCreate (FLASH0, 512, 512, 512, 0);
*     pVolDesc = dosFsDevInit ("DEV1:", pBlkDev, NULL);
* .CE
* In this case, dosFsDevInit() must be used instead of dosFsMkfs(), because
* the file system already exists on the disk and should not be
* re-initialized.  This procedure is the normal case. Making a filesystem
* should only be done after erasing the previous contents of flash memory.
*
* These same procedures apply when creating a Flash disk with rt11Fs using
* rt11FsDevInit() and rt11FsMkfs(), or creating a Flash disk with rawFs using
* rawFsDevInit().
*
* RETURNS:
* A pointer to a block device structure (BLK_DEV) or NULL if memory cannot
* be allocated for the device structure or for the Flash disk.
*
* SEE ALSO: dosFsMkfs(), dosFsDevInit(), rt11FsDevInit(), rt11FsMkfs(),
* rawFsDevInit()
*/

BLK_DEV *flashDevCreate
    (
    char	flashDevice,	/* which flash part it is */
    FAST int	bytesPerBlk,	/* number of bytes per block */
    int		blksPerTrack,	/* number of blocks per track */
    int		nBlocks,	/* number of blocks on this device */
    int		blkOffset	/* no. of blks to skip at start of device */
    )
					 

    {
    FAST FLASH_DEV	*pFlashDev;	/* ptr to created FLASH_DEV struct */
    FAST BLK_DEV	*pBlkDev;	/* ptr to BLK_DEV struct in FLASH_DEV */


    /* Set up defaults for any values not specified */

    if (bytesPerBlk == 0)
	bytesPerBlk = DEFAULT_SEC_SIZE;

    if (nBlocks == 0)
	nBlocks = DEFAULT_DISK_SIZE / bytesPerBlk;

    if (blksPerTrack == 0)
	blksPerTrack = nBlocks;


    /* Allocate a FLASH_DEV structure for device */

    pFlashDev = (FLASH_DEV *) malloc (sizeof (FLASH_DEV));
    if (pFlashDev == NULL)
	return (NULL);					/* no memory */


    /* Initialize BLK_DEV structure (in FLASH_DEV) */

    pBlkDev = &pFlashDev->flash_blkdev;

    pBlkDev->bd_nBlocks      = nBlocks;		/* number of blocks */
    pBlkDev->bd_bytesPerBlk  = bytesPerBlk;	/* bytes per block */
    pBlkDev->bd_blksPerTrack = blksPerTrack;	/* blocks per track */

    pBlkDev->bd_nHeads       = 1;		/* one "head" */
    pBlkDev->bd_removable    = FALSE;		/* not removable */
    pBlkDev->bd_retry	     = 2;		/* retry count */
    pBlkDev->bd_mode	     = O_RDWR;		/* initial mode for device */
    pBlkDev->bd_readyChanged = TRUE;		/* new ready status */

    pBlkDev->bd_blkRd	     = flashBlkRd;	/* read block function */
    pBlkDev->bd_blkWrt	     = flashBlkWrt;	/* write block function */
    pBlkDev->bd_ioctl	     = flashIoctl;	/* ioctl function */
    pBlkDev->bd_reset	     = NULL;		/* no reset function */
    pBlkDev->bd_statusChk    = NULL;		/* no check-status function */


    /* Initialize remainder of device struct */

	pFlashDev->flash_device = flashDevice;


    pFlashDev->flash_blkOffset = blkOffset;			/* block offset */


    return (&pFlashDev->flash_blkdev);
    }

/*******************************************************************************
*
* flashIoctl - do device specific control function
*
* This routine is called when the file system cannot handle an ioctl()
* function.
*
* The FIODISKFORMAT function erases the flash.
* All other requests return ERROR.
*
* RETURNS:  OK or ERROR.
*
* ARGSUSED1
*/

LOCAL STATUS flashIoctl (pFlashDev, function, arg)
    FLASH_DEV	*pFlashDev;		/* device structure pointer */
    int		function;		/* function code */
    int		arg;			/* some argument */

    {
    FAST int	status;			/* returned status value */


    switch (function)
	{
	case FIODISKFORMAT:
	    status = flashErasePart(pFlashDev->flash_device);
	    break;

	case FIODISKFORMATALL:
#ifndef MRGCL1
		PackPartPercentScale = 96/2;				/* erase takes 96%, each device takes 48% */
	    status = flashErase(FLASH3) ? OK : ERROR;
		PackPartPercentDone = 0;
		PackPercentDone = 48;
	    status = flashErase(FLASH4) ? OK : ERROR;
		PackPartPercentDone = 0;
		PackPercentDone = 96;
#else
		PackPartPercentScale = 96;
	    status = flashErase(FLASH2) ? OK : ERROR;
		PackPartPercentDone = 0;
		PackPercentDone = 96;
#endif
	    break;

	default:
	    errnoSet (S_ioLib_UNKNOWN_REQUEST);
	    status = ERROR;
	}

    return (status);
    }

/*******************************************************************************
*
* flashBlkRd - read one or more blocks from a Flash disk volume
*
* This routine reads one or more blocks from the specified volume,
* starting with the specified block number.  The byte offset is
* calculated and the Flash disk data is copied to the specified buffer.
*
* If any block offset was specified during flashDevCreate(), it is added
* to <startBlk> before the transfer takes place.
*
* RETURNS: OK if expected number of bytes actually read, ERROR otherwise.
*/

LOCAL STATUS flashBlkRd (pFlashDev, startBlk, numBlks, pBuffer)
    FAST FLASH_DEV	*pFlashDev;	/* pointer to device desriptor */
    int			startBlk;	/* starting block number to read */
    int			numBlks;	/* number of blocks to read */
    char		*pBuffer;	/* pointer to buffer to receive data */

    {
    FAST int		bytesPerBlk;	/* number of bytes per block */
    int			nbytes;


    bytesPerBlk = pFlashDev->flash_blkdev.bd_bytesPerBlk;

    /* Add in the block offset */

    startBlk += pFlashDev->flash_blkOffset;

    /* Read the block(s) */

    nbytes = flashReadPart(pFlashDev->flash_device, (UINT8*)(startBlk * bytesPerBlk),
	    (UINT8*)pBuffer, bytesPerBlk * numBlks);

    if (nbytes != (numBlks * bytesPerBlk))
	return (ERROR);

    return (OK);
    }

/*******************************************************************************
*
* flashBlkWrt - write one or more blocks to a Flash disk volume
*
* This routine writes one or more blocks to the specified volume,
* starting with the specified block number.  The byte offset is
* calculated and the buffer data is copied to the Flash disk.
*
* If any block offset was specified during flashDevCreate(), it is added
* to <startBlk> before the transfer takes place.
*
* RETURNS: OK if expected number of bytes actually written, ERROR otherwise.
*/

LOCAL STATUS flashBlkWrt (pFlashDev, startBlk, numBlks, pBuffer)
    FAST FLASH_DEV	*pFlashDev;	/* pointer to device desriptor */
    int			startBlk;	/* starting block number to write */
    int			numBlks;	/* number of blocks to write */
    char		*pBuffer;	/* pointer to buffer of data to write */

    {
    FAST int		bytesPerBlk;	/* number of bytes per block */
    int			nbytes;


    bytesPerBlk = pFlashDev->flash_blkdev.bd_bytesPerBlk;

    /* Add in the block offset */

    startBlk += pFlashDev->flash_blkOffset;

    /* Write the block(s) */

    nbytes = flashWritePart(pFlashDev->flash_device, (UINT8*)(startBlk * bytesPerBlk),
	    (UINT8*)pBuffer, bytesPerBlk * numBlks);

    return (nbytes != (numBlks * bytesPerBlk)) ? ERROR : OK;
}
