/* flashFsLib.h - header for MS-DOS media-compatible file system library */
/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Bill Randle, Tektronix, Inc. <billr@saab.cna.tek.com>
 */ 

/*
modification history
--------------------
01x,29jul93,bcr	 written


MS-DOS is a registered trademark of Microsoft Corporation.

*/

#ifndef __INCflashFsLibh
#define __INCflashFsLibh

#include "dosFsLib.h"

/*
The flash version of the dosFsLib looks very similar to the standard (disk)
version. We just tweak some of the constants to be numbers more suitable
to the program/erase characteristics/limitations of the flash memory.
*/

/* swap DEL_MARK & NAME_E5 since we can write 0's to delete but not v.v. */
#define FLASH_DEL_MARK		0x00	/* file is deleted if name[0] == 0x05 */
#define FLASH_UNUSED_MARK	0xff	/* unused dirent slot if name[0] == 0xff */

/* File Allocation Table (FAT) values */
/* simplify flash FAT by only using 16-bit block numbers */
#define	FLASH_FAT_AVAIL		0xffff	/* cluster is available */
#define	FLASH_FAT16_BAD		0x3fff	/* bad cluster marker for 16-bit FATs */
#define	FLASH_FAT16_EOF		0x7fff	/* end-of-file marker for 16-bit FATs */
#define	FLASH_FAT_DIRTY		0x0000	/* dirty cluster marker for FATs */

/* structure filled in by flashFsSizeInfo */
typedef struct {
    int total;		/* bytes on flash disk (counts boot, FAT, directory) */
    int free;		/* free bytes */
    int dirty;		/* dirty bytes */
    int active;		/* active file bytes */
} FLASH_SIZE_INFO;



#ifdef __cplusplus
extern "C" {
#endif

/* Function declarations */

#if defined(__STDC__) || defined(__cplusplus)

extern STATUS 		flashFsConfigGet (DOS_VOL_DESC *vdptr,
					DOS_VOL_CONFIG *pConfig);
extern STATUS 		flashFsConfigInit (DOS_VOL_CONFIG *pConfig,
					char mediaByte, char secPerClust,
					short nResrvd, char nFats,
					short secPerFat, short maxRootEnts,
					UINT nHidden, UINT options);
extern STATUS 		flashFsConfigShow (char *devName);
extern STATUS 		flashFsDateSet (int year, int month, int day);
extern void 		flashFsDateTimeInstall (FUNCPTR pDateTimeFunc);
extern DOS_VOL_DESC*	flashFsDevInit (char *devName, BLK_DEV *pBlkDev, DOS_VOL_CONFIG *pConfig);
extern STATUS 		flashFsInit (int maxFiles);
extern DOS_VOL_DESC*	flashFsMkfs (char *volName, BLK_DEV *pBlkDev);
extern STATUS		flashFsMkfsOptionsSet (UINT options);
extern void 		flashFsModeChange (DOS_VOL_DESC *vdptr, int newMode);
extern void 		flashFsReadyChange (DOS_VOL_DESC *vdptr);
extern STATUS 		flashFsTimeSet (int hour, int minute, int second);
extern STATUS 		flashFsVolOptionsGet (DOS_VOL_DESC *vdptr, UINT *pOptions);
extern STATUS 		flashFsVolOptionsSet (DOS_VOL_DESC *vdptr, UINT options);
extern STATUS 		flashFsVolUnmount (DOS_VOL_DESC *vdptr);
extern STATUS 		flashFsSizeInfo (char* devname, FLASH_SIZE_INFO* info);

#else	/* __STDC__ */

extern STATUS 	flashFsConfigGet ();
extern STATUS 	flashFsConfigInit ();
extern STATUS 	flashFsConfigShow ();
extern STATUS 	flashFsDateSet ();
extern void 	flashFsDateTimeInstall ();
extern DOS_VOL_DESC *	flashFsDevInit ();
extern STATUS 	flashFsInit ();
extern DOS_VOL_DESC *	flashFsMkfs ();
extern STATUS	flashFsMkfsOptionsSet ();
extern void 	flashFsModeChange ();
extern void 	flashFsReadyChange ();
extern STATUS 	flashFsTimeSet ();
extern STATUS 	flashFsVolOptionsGet ();
extern STATUS 	flashFsVolOptionsSet ();
extern STATUS 	flashFsVolUnmount ();
extern STATUS	flashFsSizeInfo ();

#endif	/* __STDC__ */


#ifdef __cplusplus
};
#endif


#endif /* __INCflashFsLibh */
