/*===== $Id: packer.c,v 1.3 1993/09/07 21:48:56 kenc Exp $ =====*/
/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Ken Coulson, Tektronix, Inc. <kenc@lollipop.cna.tek.com>
 */ 

/*************************************************************************
 *
 *  $RCSfile: packer.c,v $
 *
 *  $Revision: 1.3 $
 *
 *  Description:  Ken Coulson
 *
 *======================================================================*
 *
 *  REVISIONS:
 *  ----------
 *  Revision history available via rcs command 'rlog'.
 *
 *======================================================================*
 *  Copyright 1993 Tektronix, Inc.  All rights reserved.
 *************************************************************************/


#include <procpriority.h>
#include <flags.h>
#include "packer.h"
#include "storagehandler.h"

extern "C" {
#include "flashFsLib.h"
#include <ioLib.h>
};

#define PACK_DEVICES	0		// exchange messages to run() from packDevices() and deleteAll()
#define DELETE_ALL		1

int PackPercentDone;			// GLOBAL!!! (it was easier...)
int PackPartPercentDone;
int PackPartPercentScale;


/*=======================================================================*
 *              The one instance of the packer.                          * 
 *=======================================================================*/
#ifndef NOT_CLANDS
Packer packer;		// moved to StaticGlobals.c
#endif


/*=======================================================================*
 *                      Packer Routines.                                 * 
 *=======================================================================*/

void Packer::packDevices(int packlist)
{
	Message msg;

	clear();
	state =	PACKER_INPROGRESS;
	msg[0] = PACK_DEVICES;
	msg[1] = packlist;
	msg[2] = 0;
	inExch.send(msg);
}


void Packer::deleteAll()
{
	Message msg;

	clear();
	state = PACKER_INPROGRESS;
	msg[0] = DELETE_ALL;
	msg[1] = 0;
	inExch.send(msg);
}


int Packer::progress()
{

	//printf("PackPercentDone=%d PackPartPercentDone=%d PackPartPercentScale=%d\n", PackPercentDone, PackPartPercentDone, PackPartPercentScale);
	return PackPercentDone + ((PackPartPercentDone * PackPartPercentScale) / 100);
}


int Packer::status()
{
	return state;
}


void Packer::clear(void)
{
	state =					PACKER_IDLE;
	PackPercentDone =		0;
	PackPartPercentDone =	0;
	PackPartPercentScale =	100;
}


void Packer::packit(int packlist)
{
	int i;
	int fd;
	FLASH_SIZE_INFO info;
	int numParts = storageHandler.numdevices();
	int howmany = 0;

	// see how many devices will really be packed
	for (i=0; i<numParts; ++i) {
		if (packlist & (1<<i)) {
			flashFsSizeInfo(storageHandler.dev2name(i), &info);
			if (info.dirty != 0) ++howmany;
			else packlist &= ~(1<<i);
		}
	}
	PackPartPercentScale = 100 / howmany;

	if (storagePrint) printf("packit howmany=%d packlist=%x\n", howmany, packlist);

	// pack necessary devices
	for (i=0; i<numParts; ++i) {
		if (packlist & (1<<i)) {
			if ((fd = open(storageHandler.dev2name(i), O_RDWR, 0)) == ERROR) {
				if (storagePrint) printf("Error opening device <%d>\n", i);
				state = PACKER_ERROR;
				return;
			}
			if (ioctl(fd, FIOSQUEEZE, (void*)storageHandler.dev2name(i)) != OK) {
				if (storagePrint) printf("Error packing device %d\n", i);
				state = PACKER_ERROR;
				close(fd);
				return;
			}
			close(fd);
			PackPercentDone += PackPartPercentScale;
			PackPartPercentDone = 0;
		}
	}

	PackPercentDone = 100;
}


void Packer::initAll()
{
	int fd;
	int i;
	int numParts = storageHandler.numdevices();

	// unmount to flush buffers
	for (i=0; i<numParts; ++i) {
		fd = open(storageHandler.dev2name(i), O_RDWR, 0);
		if (fd == ERROR) {
			if (storagePrint) printf("Error opening device <%d>\n", i);
			state = PACKER_ERROR;
			return;
		}
		ioctl(fd, FIOUNMOUNT, 0);
		close(fd);
	}

	// erase all storage devices (other code will take care of percentage numbers...)
	fd = open(storageHandler.dev2name(0), O_RDWR, 0);
	if (ioctl(fd, FIODISKFORMATALL, 0) != OK) {
		if (storagePrint) printf("Packer::initAll() FIODISKFORMATALL error\n");
		state = PACKER_ERROR;
		close(fd);
		return;
	}
	close(fd);
	PackPartPercentDone = 0;
	PackPercentDone = 96;		// erase took 96% of time

	// build a filesystem on each device
	PackPartPercentScale = 0;
	PackPartPercentDone = 4 / numParts;
	for (i=0; i<numParts; i++) {
		// do a DISKINIT to rebuild file system
		fd = open(storageHandler.dev2name(i), O_RDWR, 0);
		if (ioctl(fd, FIODISKINIT, 0) != OK) {
			if (storagePrint) printf("Packer::initAll() FIODISKINIT failed on device %d\n", i);
			state = PACKER_ERROR;
			close(fd);
			return;
		}
		close(fd);
		PackPercentDone += PackPartPercentDone;
	}

	PackPercentDone = 100;
}


void Packer::run()
{
	Message msg;

	for (;;) {
		inExch.request(msg);
		switch (msg[0]) {
			case PACK_DEVICES:
				packit(msg[1]);
				if (state != PACKER_ERROR) state = PACKER_IDLE;
				break;
			case DELETE_ALL:
				initAll();
				if (state != PACKER_ERROR) state = PACKER_IDLE;
				break;
			default:
				if (storagePrint) printf("Packer::run illegal message %d\n", msg[0]);
				state = PACKER_ERROR;
				break;
		}
	}
}


Packer::Packer() : ("PACK", PACKER_PRIORITY, 2048, FALSE, TRUE), inExch("pckI", FIFO, UNRESTRICTED)
{
	clear();
}
