/*===== $Id: packer.h,v 1.2 1993/09/07 21:48:56 kenc Exp $ =====*/

#ifndef PACKER_H
#define PACKER_H
/*
 * Copyright 1993, 1994 by Tektronix, Inc. - All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Tektronix, Inc. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.
 * 
 * TEKTRONIX INCORPORATED MAKES NO REPRESENTATIONS ABOUT THE
 * SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS"
 * WITHOUT EXPRESS OR IMPLIED WARRANTY.  TEKTRONIX INCORPORATED
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO
 * EVENT SHALL TEKTRONIX INCORPORATED BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author: Ken Coulson, Tektronix, Inc. <kenc@lollipop.cna.tek.com>
 */ 

/*************************************************************************
 *
 *  $RCSfile: packer.h,v $
 *
 *  $Revision: 1.2 $
 *
 *  Description:  Flash Memory Packer Process.
 *
 *  Classes:      Packer
 *
 *  $Author: kenc $
 *
 *======================================================================*
 *
 *  REVISIONS:
 *  ----------
 *  Revision history available via 'rlog'.
 *
 *======================================================================*
 *
 *  The packer class is responsible for:
 *
 *   1>   Repacking the file system one or more flash eprom parts to
 *		  recover unused space taken up by deleted files.
 *   2>	  Provide a progress indicator to other processes.
 *
 *   In typical usage, the storage handler will call either packDevices()
 *   or deleteAll() then loop doing a pause, followed by a call to
 *   progress() to get the current completion percentage, until progress()
 *   returns -1.
 *
 *======================================================================*
 *   Copyright 1993 Tektronix, Inc.  All rights reserved.
 *************************************************************************/

#ifdef cplusplus_2_0

#include <Process.h>
#include <Message.h>
#include <Exchange.h>

#define PACKER_IDLE			0		// packer states
#define PACKER_INPROGRESS	1
#define PACKER_ERROR		2


/*=======================================================================* 
 *                       The Packer class.                               *
 *=======================================================================*/
class Packer : public Process
{
public:
	Packer(void);
	~Packer(void) {};
		//

	void clear(void);
		// set all status and percent reporting variables to idle

    int status(void);
		// returns current packer state

	int progress(void);
		// report packing progress as a percentage (0-100)
	
	void packDevices(int packlist);
		// pack one or more devices. Packlist is a bit field denoting which
		// device(s) to pack. Bit0 = device 0, etc.
	
	void deleteAll(void);
		// erases and rebuilds file system for all flash devices

private:
	Exchange inExch;
		//

	int state;
		//

	void run(void);
		//

	void initAll(void);
		// erase and reformat

	void packit(int packlist);
		// do the actual packing
};

extern Packer packer;				// instance of class is in StaticGlobals.c

#endif

extern int PackPercentDone;			/* globals from packer.c */
extern int PackPartPercentDone;
extern int PackPartPercentScale;

#endif

